% This function generates a graph with the color-coded and weighted edges using PC-stable algorithm (uses PC-stable.m).
% The 2D node placement can be enhanced using tsne embedding.
% inputs: 
% Data is the input rows: samples, columns: dimensions (nodes)
% Name is the node names
% optional inputs: 
% pvalue: for partial correlations to keep an edge (default 0.01) 
% tsne : to use tsne (default not to use tsne) 
% pcor_color: to use partial correlations for coloring edges (default not to use pcor_color)
% Taraneh Attary and Ali Ghazizadeh 2021

function g=PC_graph_plot(Data,Name,varargin)

if any(strcmpi('pvalue',varargin)), pval=varargin{find(strcmpi('pvalue',varargin))+1};else pval=0.01; end
if any(strcmpi('tsne',varargin)), tsne_flag=1;else tsne_flag=0; end
if any(strcmpi('pcor_color',varargin)), pcor_color=1;else pcor_color=0; end

%% running PC_stable algorithm and initial graph plotting
[C_unsigned , C_signed] = PC_stable(Data,pval);

c1=graph(C_unsigned,Name,'upper','OmitSelfLoops');
g=plot(c1);
g.MarkerSize=10;
%% The width of each edge
C1=triu(C_unsigned);
in3=C1>0;
C2=C1(in3);

[in1,in2]=find(C1>0);
for m=1:size(in1)
highlight(g,in1(m),in2(m),'LineWidth',10*C2(m))
end

%% color code the edges as pairwise correlation matrix 

if pcor_color
    [c1,l1]=find(C_signed>0);
    highlight(g,c1,l1,'EdgeColor','r')
else
    corr1=corr(Data);
    ind1=C1<=0;
    corr1(ind1)=0;
    [c1,l1]=find(corr1>0);
    highlight(g,c1,l1,'EdgeColor','r')
end

%% using tsne
if tsne_flag
    %% tsne for order of nodes
    fprintf('positioning nodes using tsne..\n')
    Data1=Data';
    D=pdist(Data1,'correlation');
    [Y1,eigvals] = mdscale(D,2,'criterion','sstress');
    Y = tsne(Data1,'Algorithm','exact','Distance','correlation','Exaggeration',80,'LearnRate',100,'Perplexity',10,'InitialY',Y1);
    option= statset('MaxIter',5000);
    
    g.XData=Y(:,1);
    g.YData=Y(:,2);
end


end

