% usage: Result = PC_stable(data,p-value) 
% data n*m: m is the number of nodes, n is the number of subjects
%
% OVERVIEW:
%  To plot the pc stable algorithm, should consider
%  an initially complete graph with all edges, then start to eliminate 
%  edges in each level (l). In each level, each pair of nodes (i and j) that 
%  are adjacent in the graph (C) such that |adj(C,i)\{j}|>=l has been selected. 
%  then the conditionally dependent given k (the third node) will be checked (with 
%  the use of partial correlation). if the nodes are independent of each other
%  the edge between them will be eliminated. then the new k (other nodes between i and j) 
%  will be selected and the process has been repeated. after checking  
%  all k nodes, the new pair of nodes that are adjacent such that |adj(C,i)\{j}|>=l
%  has been selected and the process repeat again until all ordered pairs of adjacent 
%  variables i and j such that |adj(C,i)\{j}|>=l and k nodes between them have been 
%  tested for conditional independence. then it goes to the next level and repeats all 
%  the processes again until for each ordered pair of adjacent nodes i,j: |adj(C,i)\{j}|<l
%  v1 written by Mohammad-Amin Fakharian 2021, v2 modified by MohammadHossein Nadian 2023.


function C = PC_stable(V,thresh)

V = V';
num = size(V,1);
N = size(V,2);
C = ones(num) .* not(diag(ones(num,1)));
C_wght = C;
l = -1;

while (l <= max(sum(C)-1))
    
    l = l + 1;
    i_omit = [];
    j_omit = [];
    
    disp(['-------------------- l = ',num2str(l),' --------------------']);
    
    for i = 1:num
        
        %         perccount(i,num);
        
        if (sum(C(i,:))<=l)
            continue;
        end
        
        ind_adj = find(C(i,:)==1);
        num_adj = length(ind_adj);
        
        for j = 1:num_adj
%              rho =1; nadian

            rho =min(1,C_wght(i,j));
			% Nadian
            adj_set = ind_adj(not(ind_adj == ind_adj(j)));
            
            if length(adj_set)<l
                continue;
            end
            
            if l == 0
                K_set = [];
                rho_old = rho;
                [rho,p] = corr(V(i,:)',V(ind_adj(j),:)');
                %                 z = log((rho+1)/(1-rho))/2;
                %                 Tn = abs(z * sqrt(N-3));
                %                 pval = 1-normcdf(Tn);
                pval = p;
                
                
                if(pval <= thresh)
                    wght = rho;
                    if abs(rho) <= abs(rho_old)
                        C_wght(i,ind_adj(j)) = wght;
                        C_wght(ind_adj(j),i) = wght;
                    end
                    continue;
                end
                
                i_omit = [i_omit, i];
                j_omit = [j_omit, ind_adj(j)];
            elseif length(adj_set) == 1
                K_set = adj_set;
            else
                K_set = nchoosek(adj_set,l);
            end
            
            for k = 1:size(K_set,1)
                
                K = K_set(k,:);
                [Rho,P] = partialcorr([V(i,:)',V(ind_adj(j),:)'],...
                    V(K,:)');
                rho_old = rho;
                rho = Rho(1,2);
                %                 z = log((rho+1)/(1-rho))/2;
                %                 Tn = abs(z * sqrt(N-3));
                %                 pval = 1-normcdf(Tn);
                pval = P(1,2);
%                 if (pval >= thresh)
%                    i
%                    j
%                    k
%                 end
                
                if(pval <= thresh)
                    wght = rho;
                    if abs(rho) <= abs(rho_old)
                        C_wght(i,ind_adj(j)) = wght;
                        C_wght(ind_adj(j),i) = wght;
                    end
                    continue;
                end
                
                i_omit = [i_omit, i];
                j_omit = [j_omit, ind_adj(j)];
                
            end
            
        end
        
    end
    
    for ii = 1:length(i_omit)
        C(i_omit(ii),j_omit(ii)) = 0;
        C(j_omit(ii),i_omit(ii)) = 0;
    end
    
end
C = C .* C_wght;
end